@@ -13,194 +13,48 @@
 // License for the specific language governing permissions and limitations under
 // the License.
 
-use std::{net, str};
-
-use crate::error::Error;
+use std::str;
 
 use super::{
     addr::Addr,
     parser::{HostExprLanguage, Pairs, Parser, Rule},
-    port_set::{Port, PortSet},
+    port_set::PortSet,
 };
 
 #[derive(Clone, Debug, PartialEq)]
-pub struct ConnectionExpr {
-    ports: PortSet,
-    addr: Option<Addr>,
-    tunnel: Option<TunnelExpr>,
-}
-
-#[derive(Clone, Debug, PartialEq)]
-pub struct TunnelExpr {
-    user: Option<String>,
-    addr: Addr,
-    ports: Option<PortSet>,
+pub enum ConnectionExpr {
+    RouteExpr(RouteExpr),
+    HostKey(String),
 }
 
 impl ConnectionExpr {
-    pub fn resolve_routes(&self) -> Result<Routes, Error> {
-        Ok(Routes {
-            inner: RouteSet::try_from_conn_expr(self)?,
-            pos: 0,
-        })
-    }
-}
-
-impl RouteSet {
-    fn try_from_conn_expr(conn_expr: &ConnectionExpr) -> Result<Self, Error> {
-        if let Some(ref tunnel) = conn_expr.tunnel {
-            let host_addr = conn_expr
-                .addr
-                .as_ref()
-                .expect("tunneling should guarantee final host address")
-                .clone();
-            Ok(RouteSet::Tunneled {
-                ssh_user: tunnel.user.clone(),
-                ssh_addr: tunnel.addr.clone(),
-                ssh_ports: tunnel.ports.clone(),
-                host_addr,
-                host_ports: conn_expr.ports.clone(),
-            })
-        } else {
-            let mut ips = match conn_expr.addr {
-                None => dns_lookup::lookup_host("localhost").map_err(|_| {
-                    Error::DomainNotFound("localhost".to_owned())
-                })?,
-                Some(Addr::Domain(ref domain)) => {
-                    dns_lookup::lookup_host(domain)
-                        .map_err(|_| Error::DomainNotFound(domain.clone()))?
-                }
-                Some(Addr::IP(ip)) => vec![ip],
-            };
-            ips.sort();
-            Ok(RouteSet::Direct {
-                ips,
-                ports: conn_expr.ports.clone(),
-            })
-        }
-    }
-}
-
-#[derive(Clone, Debug)]
-pub enum Route {
-    Direct(net::SocketAddr),
-    // Note that we let the ssh client to resolve the ssh server's address and,
-    // likewise, the ssh server to resolve to final host's address.  This way
-    // the name resolution behaves the same as it would when you debug it by
-    // hand with the actual ssh client.
-    Tunneled(TunnelOptions),
-}
-
-#[derive(Clone, Debug)]
-pub struct TunnelOptions {
-    pub ssh_user: Option<String>,
-    pub ssh_addr: Addr,
-    pub ssh_port: Option<Port>,
-    pub host_addr: Addr,
-    pub host_port: Port,
-}
-
-#[derive(Clone, Debug)]
-pub struct Routes {
-    inner: RouteSet,
-    pos: usize,
-}
-
-impl Iterator for Routes {
-    type Item = Route;
-
-    fn next(&mut self) -> Option<Self::Item> {
-        if self.pos < self.inner.len() {
-            let item = self.inner.produce(self.pos);
-            self.pos += 1;
-            Some(item)
+    pub fn try_as_route_expr(&self) -> Option<&RouteExpr> {
+        if let ConnectionExpr::RouteExpr(ref e) = *self {
+            Some(e)
         } else {
             None
         }
     }
 }
 
-#[derive(Clone, Debug)]
-enum RouteSet {
-    Direct {
-        ips: Vec<net::IpAddr>,
-        ports: PortSet,
-    },
-    Tunneled {
-        ssh_user: Option<String>,
-        ssh_addr: Addr,
-        ssh_ports: Option<PortSet>,
-        host_addr: Addr,
-        host_ports: PortSet,
-    },
+impl From<RouteExpr> for ConnectionExpr {
+    fn from(route_expr: RouteExpr) -> Self {
+        ConnectionExpr::RouteExpr(route_expr)
+    }
 }
 
-impl RouteSet {
-    fn len(&self) -> usize {
-        match self {
-            RouteSet::Direct { ips: addrs, ports } => {
-                addrs.len() * ports.as_slice().len()
-            }
-            RouteSet::Tunneled {
-                ssh_ports: None,
-                host_ports,
-                ..
-            } => host_ports.as_slice().len(),
-            RouteSet::Tunneled {
-                ssh_ports: Some(ssh_ports),
-                host_ports,
-                ..
-            } => ssh_ports.as_slice().len() * host_ports.as_slice().len(),
-        }
-    }
+#[derive(Clone, Debug, PartialEq)]
+pub struct RouteExpr {
+    pub ports: PortSet,
+    pub addr: Option<Addr>,
+    pub tunnel: Option<TunnelExpr>,
+}
 
-    fn produce(&self, ix: usize) -> Route {
-        assert!(ix < self.len());
-        match self {
-            RouteSet::Direct { ips: addrs, ports } => {
-                // Iterate resolved addresses first and given ports second
-                let ix_addr = ix % addrs.len();
-                let ix_port = ix / addrs.len();
-                Route::Direct(net::SocketAddr::new(
-                    addrs[ix_addr],
-                    ports.as_slice()[ix_port],
-                ))
-            }
-            RouteSet::Tunneled {
-                ssh_user,
-                ssh_addr,
-                ssh_ports: None,
-                host_addr,
-                host_ports,
-            } => Route::Tunneled(TunnelOptions {
-                ssh_user: ssh_user.clone(),
-                ssh_addr: ssh_addr.clone(),
-                ssh_port: None,
-                host_addr: host_addr.clone(),
-                host_port: host_ports.as_slice()[ix],
-            }),
-            RouteSet::Tunneled {
-                ssh_user,
-                ssh_addr,
-
-                ssh_ports: Some(ssh_ports),
-                host_addr,
-                host_ports,
-            } => {
-                // Iterate ssh host's ports first and final host's ports second
-                let ssh_ports = ssh_ports.as_slice();
-                let ix_ssh_port = ix % ssh_ports.len();
-                let ix_host_port = ix / ssh_ports.len();
-                Route::Tunneled(TunnelOptions {
-                    ssh_user: ssh_user.clone(),
-                    ssh_addr: ssh_addr.clone(),
-                    ssh_port: Some(ssh_ports[ix_ssh_port]),
-                    host_addr: host_addr.clone(),
-                    host_port: host_ports.as_slice()[ix_host_port],
-                })
-            }
-        }
-    }
+#[derive(Clone, Debug, PartialEq)]
+pub struct TunnelExpr {
+    pub user: Option<String>,
+    pub addr: Addr,
+    pub ports: Option<PortSet>,
 }
 
 impl str::FromStr for ConnectionExpr {
@@ -219,20 +73,26 @@ impl str::FromStr for ConnectionExpr {
                 connection_expr_from_local_connection_expr_pair(
                     top_pair.into_inner(),
                 )
+                .map(|e| e.into())
             }
             Rule::remote_connection_expr => {
                 connection_expr_from_remote_connection_expr_pair(
                     top_pair.into_inner(),
                 )
+                .map(|e| e.into())
             }
             Rule::tunneled_connection_expr => {
                 connection_expr_from_tunneled_connection_expr_pair(
                     top_pair.into_inner(),
                 )
+                .map(|e| e.into())
+            }
+            Rule::host_key_expr => {
+                Ok(ConnectionExpr::HostKey(top_pair.as_str().to_string()))
             }
             _ => unreachable!(
-                r#"grammar guarantees local, remote, or tunneled remote host \
-                   expression"#
+                r#"grammar guarantees a local, remote, or tunneled route
+                   expression to a remote host, or a host key reference"#
             ),
         }
     }
@@ -240,8 +100,8 @@ impl str::FromStr for ConnectionExpr {
 
 fn connection_expr_from_local_connection_expr_pair(
     mut pairs: Pairs<Rule>,
-) -> Result<ConnectionExpr, ParseError> {
-    Ok(ConnectionExpr {
+) -> Result<RouteExpr, ParseError> {
+    Ok(RouteExpr {
         ports: pairs
             .next()
             .expect("grammar guarantees a port set")
@@ -255,7 +115,7 @@ fn connection_expr_from_local_connection_expr_pair(
 
 fn connection_expr_from_remote_connection_expr_pair(
     mut pairs: Pairs<Rule>,
-) -> Result<ConnectionExpr, ParseError> {
+) -> Result<RouteExpr, ParseError> {
     let addr = pairs
         .next()
         .expect("grammar guarantees an address")
@@ -273,7 +133,7 @@ fn connection_expr_from_remote_connection_expr_pair(
 
 fn connection_expr_from_tunneled_connection_expr_pair(
     pairs: Pairs<Rule>,
-) -> Result<ConnectionExpr, ParseError> {
+) -> Result<RouteExpr, ParseError> {
     let (tunnel, mut pairs) = tunnel_from_pairs(pairs)?;
     let mut connection_expr = connection_expr_from_remote_connection_expr_pair(
         pairs
@@ -374,11 +234,11 @@ mod test {
     #[test]
     fn local_connection_expr_parsing() {
         let mk = |ports| {
-            Ok(ConnectionExpr {
+            Ok(ConnectionExpr::RouteExpr(RouteExpr {
                 ports: ps(ports),
                 addr: None,
                 tunnel: None,
-            })
+            }))
         };
         assert_eq!("1".parse(), mk(&[1]));
         assert_eq!("1,2".parse(), mk(&[1, 2]));
@@ -389,11 +249,11 @@ mod test {
     #[test]
     fn remote_connection_expr_parsing() {
         let mk = |addr, ports| {
-            Ok(ConnectionExpr {
+            Ok(ConnectionExpr::RouteExpr(RouteExpr {
                 ports: ps(ports),
                 addr: Some(addr),
                 tunnel: None,
-            })
+            }))
         };
         assert_eq!("1.2.3.4:1,2-3".parse(), mk(ip4(1, 2, 3, 4), &[1, 2, 3]));
         assert_eq!(
@@ -410,7 +270,7 @@ mod test {
                   tunnel_ports: &[u16],
                   server_addr,
                   server_ports| {
-            Ok(ConnectionExpr {
+            Ok(ConnectionExpr::RouteExpr(RouteExpr {
                 ports: ps(server_ports),
                 addr: Some(server_addr),
                 tunnel: Some(TunnelExpr {
@@ -418,7 +278,7 @@ mod test {
                     addr: tunnel_addr,
                     ports: maybe_ps(tunnel_ports),
                 }),
-            })
+            }))
         };
         assert_eq!(
             "1.2.3.4:5.6.7.8:9".parse(),
@@ -485,4 +345,11 @@ mod test {
             mk(Some("a"), dom("b.c.d."), &[1], dom("e.f.g."), &[2])
         );
     }
+
+    #[test]
+    fn host_key_expr_parsing() {
+        let mk = |key: &str| Ok(ConnectionExpr::HostKey(key.to_owned()));
+        assert_eq!("x".parse(), mk("x"));
+        assert_eq!("my_prod_host_1".parse(), mk("my_prod_host_1"));
+    }
 }
