@@ -23,8 +23,11 @@
 
 pub mod cli;
 pub mod conn_expr;
+pub mod host_options;
+pub mod hosts_files;
 pub mod nrepl;
 pub mod outputs;
+pub mod routes;
 pub mod socket;
 pub mod sources;
 
